function [logprob] = cal_ac_logprob(x,sr,fmin,bpo,q,n,ftype,nchs,maxlags)
%function [logprob] = cal_ac_logprob(x,sr,fmin,bpo,q,n,ftype,nchs,maxlags)
%
% Calculate pitch observation in log scale using normalized subband autocorrelation.
%
% INPUT:
%         x - the input signal
%         sr - the sampling rate
%         fmin - the center frequency of the minimum frequency band
%         bpo - band-per-octave
%         q - the constant Q-factor of the filter
%         n - the order of filter
%         ftype - filter type
%         nchs - number of subband channels
%         maxlags - maximum lag (default: sr*0.025 = 400)
%
% OUTPUT:
%         logprob - pitch observation in log scale
%
% [2012-07-28] Byung Suk Lee bsl@ee.columbia.edu

if nargin < 3
if ~exist('fmin','var'), fmin = 100; end;
if ~exist('bpo','var'), bpo = 16; end;
if ~exist('q','var'), q = 8; end;
if ~exist('n','var'), n = 2; end;
if ~exist('ftype','var'), ftype = 2; end;
if ~exist('nchs','var'), nchs = 48; end;
if ~exist('maxlags','var'), maxlags = 400; end;
end
% Design Lyon-Slaney filters
[b2,a2,t2,w2,cf] = bpfiltbank(sr,fmin,bpo,nchs,q,n,ftype);  % type 2 is LySla

% Use only mono channel
if size(x,2) > 1, x = x(:,1); end;

%%
nfrms = 1 + max(floor((size(x,1) - 0.025*sr)/(0.01*sr)),0);
lags = 1:maxlags;
[lptable, uftable] = make_lptable(lags,cf,sr);
logprob = zeros(maxlags,nfrms);

%% Dithering
x = dithering(x);
xlen = length(x);
n_s = 5;
fps = 100;

%% calculate sbpca features for the audio at least the length of the analysis window, 25 ms
disp(['Autocorrelogram (maxlags:',num2str(maxlags),', nchs:',num2str(nchs),')'])
if xlen >= 0.025*sr

buf_len = round(0.1*sr); % buffer length of front and end of slice
nslice = max(ceil(length(x) / (n_s * sr))-1,1);
%% feature calculation on slices of length n_s sec
for islice = 1:nslice
    ids = n_s*sr*(islice-1) - buf_len * (islice > 1) + 1; % front buffer, except the first slice
    tide = min( n_s*sr*islice + buf_len, xlen);
    if islice == nslice
        ide = xlen;
    else
        ide = tide; % end buffer, always
    end
    disp(['(',num2str(islice),'/',num2str(nslice),') Start: ',num2str(ids),' End: ',num2str(ide)]);
    xx = x(ids:ide);

    [tmpac] = sub_cal_ac(xx,sr,b2,a2,t2);

    % cut frount and end buffer results
    tmpac = tmpac(:,:,(1+10*(islice > 1)):(end-8*(islice < nslice)));
    if islice == nslice
        lac = nfrms - fps*n_s * (nslice - 1);
    else
        lac = size(tmpac,3);
    end
    disp(['islice: ',num2str(islice),' len_tmpac: ',num2str(size(tmpac,3)),' lac: ',num2str(lac)]);
    for j = 1:lac
    % cross-channel integration
    [sch_org,sp_org] = selectChanlsPeaks(tmpac(:,:,j), lags);
    logprob(:,j + n_s*fps*(islice - 1))= logpitchprob(sch_org, sp_org, lags, lptable, uftable)';
    end
end


else
    err_txt = 'Not calculating features: ';
    err_msg = 'Input audio is shorter than the analysis window, 25 ms.';
    disp([err_txt,err_msg]);
    disp(['Zero features of ',num2str(nfrms),' frames are saved.']);
    ac = zeros(nchs,maxlags,nfrms);
    for j = 1:nfrms
    % cross-channel integration
    [sch_org,sp_org] = selectChanlsPeaks(ac(:,:,j), lags);
    logprob(:,j)= logpitchprob(sch_org, sp_org, lags, lptable, uftable)';
    end
end


function [ac, x, acf, acf_engy] = sub_cal_ac(x,sr,b2,a2,t2,mapping)
% function [ac, x, acf, acf_engy] = sub_cal_ac(x,sr,b2,a2,t2,mapping)
%
% Calculate normalized subband autocorrelation. If 'mapping' exists, do
% post-processing using k-dim PCA dimensionality reduction.
%
% INPUT:
%         x - The input signal
%         sr - The sampling rate
%         b2 - B coefficients to IIR filterbank
%         a2 - A coefficients to IIR filterbank
%         t2 - Delay coefficients to IIR filterbank
%         mapping - (nchs x 1) cell containing subband PCA mapping (maxlags x kdim)
%
% OUTPUT:
%         ac - normalized subband autocorrelation
%                  (nchs X maxlags X nfrms)
%              if 'mapping' exists, do k-dim PCA dimensionality reduction
%                  (nchs X kdim X nfrms)
%              nfrms = 1 + max(floor((size(x,1) - 0.025*sr)/(0.01*sr)),0)
%         x - the input signal (one channel)
%         acf - subband autocorrelation (not used for longer audio)
%         acf_engy - energy of subband autocorrelation (not used for longer audio)
%

flag_mapping =  exist('mapping','var') && ~isempty(mapping);

if flag_mapping, maxlags = size(mapping{1},1); else maxlags = sr*0.025; end;
if flag_mapping, kdim = size(mapping{1},2); else kdim = maxlags; end;
nchs = length(t2);

margin = floor(0.04*sr);
nfrms = 1 + max(floor((size(x,1) - 0.025*sr)/(0.01*sr)),0);

if size(x,1) < 0.025*sr
    
    acf = zeros(nchs,maxlags,1);
    acf_engy = ones(nchs,maxlags,1);

else
    x = [x;zeros(margin,1)];

    % cochlear filtering
    x = x(:)';
    e = filterbank(b2,a2,x,1,0,t2);

    [c1, s1, lags] = autocorrelogram(e, sr, maxlags); % multichannel autocorrelation

    acf = c1(:,:,1:nfrms);
    acf_engy = s1(:,:,1:nfrms);
end

ac = acf ./ (acf_engy + (acf_engy == 0));

if flag_mapping
    kac = zeros(nchs,kdim,nfrms);
    for c = 1:nchs
        tmpac = shiftdim(ac(c,:,:),1);
        %% To ignore frames that contain NaN elements
        sumtmpac = sum(tmpac);
        tmpac(:,isnan(sumtmpac)) = 0;
        kac(c,:,:) = mapping{c}' * tmpac;
    end
    ac = kac;
end

