function [path,posterior,zobslik] = pitch_tracking(obslik, prct, maxlags, transmat, prior)
% VITERBI Find the most-probable (Viterbi) path through the HMM state trellis.
% path = viterbi(prior, transmat, obslik)
%
% Inputs:
% prior(i) = Pr(Q(1) = i)
% transmat(i,j) = Pr(Q(t+1)=j | Q(t)=i)
% obslik(i,t) = Pr(y(t) | Q(t)=i)
%
% Outputs:
% path(t) = q(t), where q1 ... qT is the argmax of the above expression.
% delta(j,t) = prob. of the best sequence of length t-1 and then going to state j, and O(1:t)
% psi(j,t) = the best predecessor state, given that we ended up in state j at t
%
% [2011-02-09] Byung Suk Lee
% Modified: recovered the probability considering no-pitch state

%%
scaled = 1;
if nargin < 2
    prct = 50;
end

%%
zobslik = prctile(obslik, prct);
%zobslik = mean(obslik) + .1*std(obslik,0,1);
%zobslik = zeros(1,342);

%% recover log probability considering no-pitch state
[obslik,zobslik] = recprob(obslik, zobslik);

%%
% obslik = [zobslik;obslik];

%%
[Q, T] = size(obslik);

%%
if nargin < 4
    transmat = log([(1-0.075), 0.075/(Q-1); 0.0079, (1-0.0079)]);
%     transmat = log([(1-0.075), 0.075/(Q-1); 0.0079, (1-0.0079)/(Q-1)]);
end

if nargin < 5
    prior = ones(Q,1);
%     prior = [transmat(1,1),transmat(1,2)*ones(1,Q-1)]';
end

if nargin < 3
    maxlags = 400;  
end

lags = [1:maxlags];
delta = zeros(Q,T); % max sum of log-probability for each time frame
psi = zeros(Q,T); % argmax{state} with max probability for each time frame
path = zeros(1,T);
scale = ones(1,T);

have_viterbi_helper = (exist('viterbi_path_LOG_helper') == 3);

if ~have_viterbi_helper && exist('viterbi_path_LOG_helper.c','file')
  % Attempt to compile mex
  disp('Attempting to compile viterbi_path_LOG_helper...');
  try
    mypwd = pwd();
    cd(fileparts(which('pitch_tracking')));
    mex('viterbi_path_LOG_helper.c');
    cd(mypwd);
  catch me
    disp('...failed');
    % ignore
  end
  have_viterbi_helper = (exist('viterbi_path_LOG_helper') == 3);
end

%             [delta(j,t), psi(j,t)] = max(delta(:,t-1) + [transmat(1,2),ptchDymnonly(j-1, lags)]');
%             [delta(j,t), psi(j,t)] = max(delta(:,t-1) + [transmat(1,2),ptchDymnonly(j-1,lags,transmat(2,2))]');
            % to take care of the transition probability from 1-pitch-to-1-pitch state by adding transmat(2,2)/(Q-1)
%function y = ptchDymnonly(bias, lags)
%n = (lags - bias);
%y = exp(-abs(n-0.4)/2.4);
%y = log(y/sum(y));
pdo = repmat(lags,Q-1,1) - repmat([1:Q-1]',1,length(lags));
pdo = exp(-abs(pdo-0.4)/2.4);
pdo = log(pdo./repmat(sum(pdo,2),1,Q-1));
newtransmat = [ [transmat(1,1),transmat(2,1)*ones(1,Q-1)]', ...
                [transmat(1,2)*ones(length(lags),1),transmat(2,2)/(Q-1) + pdo]'];


if have_viterbi_helper
  [delta, psi, scale] = viterbi_path_LOG_helper(obslik, prior, newtransmat, scaled);
else

  disp('Using plain MAT...');
  %%
  t=1;
  delta(:,t) = prior + obslik(:,t);
  if scaled
      delta(:,t) = (delta(:,t) - mean(delta(:,t)));
  end
  psi(:,t) = 0; % arbitrary value, since there is no predecessor to t=1
  %%
  for t=2:T   
      for j=1:Q
        [delta(j,t), psi(j,t)] = max(delta(:,t-1) + newtransmat(:,j));
        delta(j,t) = delta(j,t) + obslik(j,t);
      end
      if scaled
        delta(:,t) = (delta(:,t) - mean(delta(:,t)));
      end
  end

end

%%
[posterior(T), path(T)] = max(delta(:,T));
for t=T-1:-1:1
    path(t) = psi(path(t+1),t+1);
    posterior(t) = delta(path(t),t);
end
path = path -1;



% ------------------------------------------------------------------------------------
function y = ptchDymnonly(bias, lags)
n = (lags - bias);
y = exp(-abs(n-0.4)/2.4);
y = log(y/sum(y));
% % ------------------------------------------------------------------------------------
% function y = ptchDymnonly(bias, lags, logprob)
% p = exp(logprob); % to take care of the transition probability
% n = (lags - bias);
% y = exp(-abs(n-0.4)/2.4);
% y = log(y/sum(y)/p);


% ------------------------------------------------------------------------------------
% to recover log probability with no-pitch state
function [reclogprob,reczlogprob] = recprob(logprob, zerologprob)
prob = exp([zerologprob;logprob]);
[Q,T] = size(prob);
reclogprob = normalise(log(prob./repmat(sum(prob),Q,1)));
reczlogprob = reclogprob(1,:);
% reclogprob = reclogprob(2:Q,:);

if 0
    figure, colormap(1-gray); subplot(211), imagesc(prob); axis xy; colorbar;
    subplot(212), imagesc(reclogprob); axis xy; colorbar;
end
